<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Controlador por defecto del sistema
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
class CoreController extends Controller
{
	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************

	/**
	 * Carga la tarea desde el directorio tasks
	 * 
	 * @param string $name
	 * @param array $params
	 * @return void
	 * @magic
	 */
	public function __call( $name, $params )
	{
		trigger_error( 'Se intenta ejecutar la tarea '.$name, E_USER_NOTICE );
		
		if( file_exists( TASKS.'/'.$name.'.php' ) )
		{
			$mdview = $this->getView();
			$_buffer = Buffer::getInstance();
			
			$_buffer->start( 'salida' );
			include TASKS.'/'.$name.'.php';
			$_buffer->end();
		}
		else
		{
			trigger_error( 'La tarea '.$name.' no existe para el controlador '.get_class($this), E_USER_WARNING );
			$this->error();
		}
	}
	
	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************
	
	/**
	 * (non-PHPdoc)
	 * @see include/lib/core/Controller#display()
	 */
	protected function display()
	{
		$_section = Registry::get( 'section' );
		
		$mdview = $this->getView();
		$_buffer = Buffer::getInstance();
		$_buffer->start( 'salida' );
		
		if( file_exists( TASKS.'/'.$_section.'.php' ) )
		{
			include TASKS.'/'.$_section.'.php';
		}
		
		$_buffer->end();
	}
	
	/**
	 * (non-PHPdoc)
	 * @see include/lib/core/Controller#error()
	 */
	protected function error()
	{
		$view = $this->getView();
		if( $view instanceof HTMLView )
		{
			$view->setPage( 'error' );
		}
		else
		{
			trigger_error( 'La vista utilizada no es heredada de HTMLView', E_USER_NOTICE );
		}
		
		$buffer = Buffer::getInstance();
		
		$view->addVar( 'system_warning', $buffer->getContent( 'e_warning', false ) );
		$view->addVar( 'system_notice', $buffer->getContent( 'e_notice', false ) );
	}
}
 
?>
