<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Información básica de la sección
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
class Section
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************
	
	/**
	 * Identificador de la seccion
	 * @var string
	 */
	private $id;
	
	/**
	 * Nombre de la seccion
	 * @var string
	 */
	private $name;
	
	/**
	 * Controlador de la seccion
	 * @var string
	 */
	private $controller;
	
	/**
	 * Tarea por defecto de la seccion
	 * @var string
	 */
	private $task;
	
	/**
	 * Nivel de permisos de la seccion
	 * @var int
	 */
	private $level;
	
	/**
	 * Tipo de sección
	 * @var string
	 */
	private $type;
	
	/**
	 * Grupos que estan permitidos en la sección
	 * @var string
	 */
	private $groups;
		
	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************
	
	/**
	 * Constructor de la clase
	 * 
	 * @param string $id
	 * @param string $name
	 * @param int[optional] $level
	 * @param string[optional] $controller
	 * @param srting[optional] $task
	 * @param string[optional] $type por defecto es 'common'
	 * @param string[optional] $groups por defecto es * (todos)
	 * @return void
	 * @magic
	 */
	public function __construct( $id, $name, $level = '', $controller = '', $task = '', $type = '', $groups = '' )
	{
		$this->id = $id;

		$this->name = $name;
		
		$this->level = ( $level == '' )?LVL_ANON:$level;
		
		$this->controller = ( $controller == '' )?Registry::get( 'DEFAULT_CONTROLLER' ):$controller;
		
		$this->task = $task;
		
		$this->type = ($type == '' )?'common':$type;
		
		$this->groups = ($groups == '')?'*':$groups;
	}
	
	//**********************************************************************************
	// Métodos para el manejo de variables
	//**********************************************************************************
	
	/**
	 * Obtiene la variable seleccionada
	 * @param string $name
	 * @return mixed
	 */
	public function get( $name )
	{
		return $this->$name;
	}
	
	/**
	 * Modifica la variable seleccionada
	 * @param string $name
	 * @param mixed $value
	 * @return void
	 */
	public function set( $name, $value )
	{
		$this->$name = $value;
	}
}
 
?>
