<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Almacena las secciones del sistema
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
class SectionTree
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************

	/**
	 * Secciones del sistema
	 * @var array
	 * @static
	 */
	private static $sections;

	
	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************

	/**
	 * Agrega una seccion al sistema
	 * @param string $id
	 * @param string $name
	 * @param string $level
	 * @param string[optional] $controller
	 * @param string[optional] $task
	 * @param string[optional] $type
	 * @param string[optional] $groups
	 * @return void
	 * @static
	 */
	public static function addSection( $id, $name, $level, $controller = '', $task = '', $type = '', $groups = '' )
	{
		if( $controller == '' )
		{
			$controller = Registry::get( 'DEFAULT_CONTROLLER' );
		}
		
		self::$sections[ $id ] = new Section( $id, $name, $level, $controller, $task, $type, $groups );
	}
	
	/**
	 * Obtiene el controlador de la seccion
	 * @param string $section
	 * @return Controller
	 * @static
	 */
	
	public static function getControllerFrom( $id )
	{
		$section = self::$sections[ $id ];
		
		if( !($section instanceof Section) )
		{
			trigger_error( 'Se intento obtener el controlador de la seccion inexistente '.$id, E_USER_WARNING );
			
			$controller = Registry::get( 'DEFAULT_CONTROLLER' ).'Controller';
			return new $controller( 'unexistent' );
		}
		
		$controller = $section->get( 'controller' ).'Controller';
		
		if( !class_exists($controller) || !is_subclass_of( $controller, 'Controller' ) )
		{
			if( file_exists( CONTROLLERS.'/class.'.strtolower($controller).'.php' ) )
			{
				include_once CONTROLLERS.'/class.'.strtolower($controller).'.php';
			}
			
			if( !class_exists($controller) || !is_subclass_of( $controller, 'Controller' ) )
			{
				$controller = Registry::get( 'DEFAULT_CONTROLLER' ).'Controller';
			}
		}
		
		return new $controller( $section->get('task'), $id, $id );
	}
	
	/**
	 * Inicializa las variables de la seccion activa
	 * @param string $section
	 * @return void
	 * @static
	 */
	public static function initSection( $id )
	{
		if(  isset(self::$sections[$id]) && self::$sections[$id] instanceof Section )
		{
			$section = self::$sections[ $id ];
			Registry::set( 'section', $id );
			Registry::set( 'unexistent', false );
		}
		else
		{
			trigger_error( 'Seccion '.$id.' no existe', E_USER_WARNING );// @todo localizar mensaje de error
			$section = self::$sections[Registry::get( 'ERROR_SECTION' )];
			Registry::set( 'section', Registry::get( 'ERROR_SECTION' ) );
			Registry::set( 'unexistent', true );
		}
		
		Registry::set( 'section_name', $section->get('name') );
		Registry::set( 'level', $section->get( 'level' ) );
		Registry::set( 'section_type', $section->get('type') );
		Registry::set( 'allowed_groups', $section->get('groups') );
	}
	
	/**
	 * Carga las secciones a partir de un archivo XML
	 * @param string $file
	 * @return boolean
	 * @static
	 */
	public static function loadFromXML( $file )
	{
		if( !file_exists( $file ) )
		{
			trigger_error( 'No existe el archivo de configuración de secciones del sistema', E_USER_NOTICE ); // @todo localizar contenido
			return false;
		}
		
		$xml = simplexml_load_file( $file );
		
		foreach( $xml->section as $section )
		{
			$controller = isset($section['controller'])?$section['controller']:'';
			$task = isset($section['task'])?$section['task']:'';
			$type = isset($section['type'])?$section['type']:'';
			$groups = isset($section['groups'])?$section['groups']:'';
			
			self::addSection( $section['id'].'', $section['name'].'', intval($section['level']), $controller.'', $task.'', $type.'', $groups.'' );
		}
		
		return true;
	}
}
 
?>
