<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Clase para el manejo de sesiones
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */

class Session
{	
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************

	/**
	 * Identificador de la aplicación actual
	 * @var string
	 * @static
	 */
	private static $app_id;
	  	
   	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************
	
   	/**
   	 * Desbloquea el seguro de la variable proporcionada
   	 * @param string $var
   	 * @return void
   	 * @static
   	 */
   	private static function unlock( $var )
   	{
   		if( isset( $_SESSION['locks-'.self::$app_id] ) )
   		{
   			$locks = $_SESSION['locks-'.self::$app_id];
   		}
   		else
   		{
   			return;
   		}
   		
   		if( empty($locks) )
   		{
   			//trigger_error('No se han definido seguros de sesión', E_USER_NOTICE); //@todo Localizar contenido
   			return;
   		}
   		
   		foreach( $locks as $key => $value )
   		{
   			$array = $_SESSION['locked-'.$key.'-'.self::$app_id];
   			if( in_array( $var, $array ) )
   			{
   				$_SESSION['locks-'.self::$app_id][$key] = false;
   			}
   		}
   	}

	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************
	
	/**
	 * Inicializa la sesión
	 * @return void
	 * @static
	 */
	public static function init()
	{
		session_start();
		self::$app_id = $_SERVER['PHP_SELF'];				
	}
	
   	/**
   	 * Método para accesar a las propiedades del objeto
   	 * @param string $var
   	 * @return mixed
   	 * @static
   	 */
	public static function get( $var )
   	{
   		if( isset($_SESSION[$var.'_'.self::$app_id] ))
   		{
   			return $_SESSION[$var.'_'.self::$app_id];
   		}
   		else
   		{
   			return NULL;
   		}   		
   	}
   	
   	/**
   	 * Método para modificar las propiedades del objeto
   	 * @param string $var
   	 * @param mixed $value
   	 * @return void
   	 * @static
   	 */
   	public static function set( $var , $value )
   	{
		$_SESSION[$var.'_'.self::$app_id] = $value;
		self::unlock( $var );		
   	}
   	
   	/**
   	 * Bloquea la modificación de las variables seleccionadas
   	 * @param string $key
   	 * @param array $array
   	 * @return void
   	 * @static
   	 */
   	public static function lock( $key, $array )
   	{
   		$_SESSION['locks-'.self::$app_id][$key] = true;
   		$_SESSION['locked-'.$key.'-'.self::$app_id] = $array;
   	}
   	
   	/**
   	 * Comprueba si las variables bloqueadas no han sido modificadas
   	 * @param string $key
   	 * @return boolean true si el seguro sigue activo
   	 * @static
   	 */
   	public static function checkLock( $key )
   	{
   		if( isset($_SESSION['locks-'.self::$app_id][$key]) )
   		{
   			return $_SESSION['locks-'.self::$app_id][$key];
   		}
   		else
   		{
   			return false;
   		}
   	}
}

?>
