<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Clase estática para acceder seguramente a los datos del usuario activo
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
class User
{
	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************
	
	/**
	 * Devuelve de forma segura una variable de usuario
	 * @param string $data
	 * @return mixed
	 * @static
	 */
	public static function get( $data )
	{
		if( !Session::checkLock( 'userdata' ))
		{
			trigger_error( 'Se ha violado el seguro de variables de usuario', E_USER_NOTICE ); //@todo localizar contenido
			
			$usermanager = Registry::get( 'usermanager' );
			if( $usermanager instanceof UserManager )
			{
				$usermanager->logout();
			}
			else
			{
				return '';
			}
		}
		
		return Session::get( $data );
	}
	
	/**
	 * Autentica el nivel del usuario contra el nivel de la sección
	 * @return boolean
	 * @static
	 */
	public static function authenticate()
	{
		$user_level = self::get( 'level' );
		$section_level = Registry::get( 'level' );
		
		if( $user_level >= $section_level )
		{
			$allowed = Registry::get( 'allowed_groups' );
			
			if( $allowed == '*' )
			{
				return true;
			}
			
			$groups = explode( ',', $allowed );
			
			foreach( $groups as $group )
			{
				if( self::inGroup( trim($group) ) )
				{
					return true;
				}
			}
			
			return false;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Comprueba si el usuario activo es parte del grupo seleccionado
	 * @param string $group
	 * @return boolean
	 * @static
	 */
	public static function inGroup( $group )
	{
		$groups = self::get( 'groups' );
		if( in_array( $group, $groups ) )
		{
			return true;
		}
		else
		{
			return false;
		}
	}
}
 
?>
