<?php

/*
 * Copyright 2009 Andrés Javier López
 *
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * Clase manejadora de usuarios
 *
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */

class UserManager
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************

	/**
	 * Campos adicionales para usuarios
	 * @var array
	 */
	private $data = array();

	/**
	 * Nombre de la tabla en la base de datos
	 * @var string
	 */
	protected $user_table;

	/**
	 * Nombre de la tabla de grupos
	 * @var string
	 */
	protected $group_table;

	/**
	 * Nombre de la tabla intermedia entre usuarios y grupos
	 * @var string
	 */
	protected $join_table;
	
	/**
	 * Perfil de conexión utilizado
	 * @var string
	 */
	private $connection_profile;

	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************

	/**
	 * Constructor de la clase
	 * @return void
	 * @final
	 * @magic
	 */
	final public function __construct()
	{
		$this->user_table = 'MD_USERS';
		$this->group_table = 'MD_GROUPS';
		$this->join_table = 'MD_USERS_GROUPS';
		$this->addDataField( 'USER_EMAIL' );
	}
	
	//**********************************************************************************
	// Métodos para el manejo de variables	
	//**********************************************************************************
	
	/**
	 * Modifica el perfil de conexión usado por los usuarios
	 * @param string $profile
	 * @return void
	 */
	public function setConnectionProfile( $profile )
	{
		$this->connection_profile = $profile;
	}

	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************

	/**
	 * Inicia sesión del usuario
	 * @param string $user
	 * @param string $password
	 * @return boolean
	 * @final
	 */
	final public function login( $user, $password )
	{
		$db = $this->getDatabase();		

		$user = $db->evalSQL( $user );
		$password = md5( $password );

		$query = 'SELECT `USER_SCREENNAME` AS `screenname`,  `USER_LEVEL` AS `level`'.$this->otherData().' FROM '.$this->user_table.' WHERE `USER_ID` = "'.$user.'" AND `USER_PASSWORD` = "'.$password.'";';
		$result = $db->query( $query );

		if( !$result )
		{
			trigger_error( 'No se pudo realizar la consulta a la base de datos de usuarios', E_USER_NOTICE ); //@todo localizar contenido
			return false;
		}

		if( $db->numRows($result) > 0 )
		{
			$data = $db->fetch( $result );

			$this->storeData( 'user_id', $user );
			$this->storeData( 'screenname', $data->screenname );
			$this->setLevel( $data->level );

			$query = 'SELECT `GROUP_NAME` AS `group` FROM `'.$this->group_table.'` NATURAL JOIN `'.$this->join_table.'` WHERE `USER_ID` = "'.$user.'";';
			$result = $db->query( $query );
			$groups = array();
			if( $result )
			{
				while( $row = $db->fetch( $result ) )
				{
					$groups[] = $row->group;
				}
			}

			$this->storeData( 'groups', $groups );

			$this->storeAllData( $data );
			$this->lockData();
		}
		else
		{
			return false;
		}

		return true;
	}

	/**
	 * Finaliza la sesión del usuario
	 * @return void
	 * @final
	 */
	final public function logout()
	{
		$this->clearAllData();

		$this->storeData( 'user_id', '' );
		$this->storeData( 'screenname', 'Anónimo' ); //@todo localizar contenido
		$this->storeData( 'groups', array() );
		$this->setLevel( LVL_ANON );
		$this->lockData();
	}

	/**
	 * Agrega un usuario a la base de datos
	 * @param string $user_id
	 * @param string $screenname
	 * @param string $password
	 * @param int $level
	 * @param array $data
	 * @return boolean
	 */
	public function createUser( $user_id, $screenname, $password, $level, $data )
	{
		$db = $this->getDatabase();

		$password = md5( $password );
		
		if( $level < 1 )
		{
			$level = 1;
		}

		$fields = ' `USER_ID`, `USER_SCREENNAME`, `USER_PASSWORD`, `USER_LEVEL`';
		$values = ' "'.$db->evalSQL($user_id).'", "'.$db->evalSQL($screenname).'", "'.$password.'", "'.$db->evalSQL($level).'"';
		foreach( $this->data as $field )
		{
			$value = $data[$field];
			$fields .= ', `'.$field.'`';
			$values .= ', "'.$db->evalSQL($value).'"';
		}

		$query = 'INSERT INTO '.$this->user_table.' ( '.$fields.' ) VALUES ('.$values.');';

		if( !$db->query( $query ) )
		{
			return false;
		}

		return true;
	}

	/**
	 * Elimina a un usuario de la base de datos
	 * @param string $user_id
	 * @return boolean
	 */
	public function deleteUser( $user_id )
	{
		$query = 'DELETE FROM `'.$this->user_table.'` WHERE `USER_ID` = "'.$user_id.'" LIMIT 1;';

		$db = $this->getDatabase();
		if( !$db->query( $query ) )
		{
			return false;
		}

		return true;
	}

	/**
	 * Agrega un usuario a uno o más grupos
	 * @param string $user_id
	 * @param string $group_string
	 * @return boolean
	 */
	public function addToGroup( $user_id, $group_string )
	{
		$db = $this->getDatabase();

		$groups = explode( ',', $group_string );

		foreach( $groups as $group )
		{
			$group = trim($group);

			$query = 'SELECT `GROUP_ID` FROM '.$this->group_table.' WHERE `GROUP_NAME` = "'.$group.'";';
			$result = $db->query( $query );

			if( $result )
			{
				if( $db->numRows( $result ) > 0 )
				{
					$group_id = $db->result( $result, 'GROUP_ID' );

					$query = 'INSERT IGNORE INTO `'.$this->join_table.'` (`USER_ID`, `GROUP_ID` ) VALUES ( "'.$user_id.'", '.$group_id.' );';
					if( !$db->query( $query ) )
					{
						return false;
					}
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		return true;
	}

	/**
	 * Agrega un grupo de usuarios
	 * @param string $group
	 * @return boolean
	 */
	public function addGroup( $group )
	{
		$db = $this->getDatabase();
		$query = 'INSERT INTO `'.$this->group_table.'` (`GROUP_NAME`) VALUES ( "'.$group.'" );';
		if( $db->query($query) )
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Elimina a un usuario del grupo específicado
	 * @param string $user_id
	 * @param string $group
	 * @return boolean
	 */
	public function removeFromGroup( $user_id, $group )
	{
		$db = $this->getDatabase();

		$query = 'SELECT `GROUP_ID` FROM `'.$this->group_table.'` WHERE `GROUP_NAME` = "'.$group.'";';
		$result = $db->query( $query );
		if( $result )
		{
			if( $db->numRows($result) > 0 )
			{
				$group_id = $db->result( $result, 'GROUP_ID' );
				$query = 'DELETE FROM `'.$this->join_table.'` WHERE `USER_ID` = "'.$user_id.'" AND `GROUP_ID` = '.$group_id.';';
				if( $db->query($query) )
				{
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Desvincula a los usuarios de un grupo y luego elimina el grupo
	 * @param string $group
	 * @return boolean
	 */
	public function removeGroup( $group )
	{
		$db = $this->getDatabase();

		$query = 'SELECT `GROUP_ID` FROM `'.$this->group_table.'` WHERE `GROUP_NAME` = "'.$group.'";';
		$result = $db->query( $query );
		if( $result )
		{
			if( $db->numRows($result) > 0 )
			{
				$group_id = $db->result( $result, 'GROUP_ID' );
				$query = 'DELETE FROM `'.$this->join_table.'` WHERE `GROUP_ID` = '.$group_id.';';
				if( $db->query($query) )
				{
					$query = 'DELETE FROM `'.$this->group_table.'` WHERE `GROUP_ID` = '.$group_id.';';
					if( $db->query($query) )
					{
						return true;
					}
				}
			}
		}

		return false;
	}

	/**
	 * Devuelve una lista de los grupos de usuarios existentes
	 * @return array
	 */
	public function getGroups()
	{
		$db = $this->getDatabase();

		$groups = array();
		$query = 'SELECT `GROUP_ID`, `GROUP_NAME` FROM `'.$this->group_table.'`;';
		$result = $db->query( $query );
		if( $result )
		{
			while( $row = $db->fetch($result) )
			{
				$groups[$row->GROUP_ID] = $row->GROUP_NAME;
			}
		}

		return $groups;
	}
	
	/**
	 * Devuelve la lista de todos los usuarios con sus datos
	 * @return array
	 */

	public function getUsers()
	{
		$query = 'SELECT `user`.*, GROUP_CONCAT( `g`.`GROUP_NAME` ) AS `groups` FROM `'.$this->user_table.'` AS `user` LEFT JOIN `'.$this->join_table.'` USING( `USER_ID` ) LEFT JOIN `'.$this->group_table.'` AS `g` USING(`GROUP_ID` ) GROUP BY `USER_ID`;';

		$db = $this->getDatabase();
		$result = $db->query( $query );

		$users = array();

		if( $result )
		{
			if( $db->numRows($result) > 0 )
			{
				while( $row = $db->fetch( $result, Database::F_ASSOC ) )
				{
					$users[] = $row;
				}
			}
		}

		return $users;
	}

	/**
	 * Devuelve la información del usuario
	 * @param string $user_id
	 * @return array
	 */
	public function getUserData( $user_id )
	{
		$query = 'SELECT `user`.*, GROUP_CONCAT( `g`.`GROUP_NAME` ) AS `groups` FROM `'.$this->user_table.'` AS `user` LEFT JOIN `'.$this->join_table.'` USING( `USER_ID` ) LEFT JOIN `'.$this->group_table.'` AS `g` USING(`GROUP_ID` ) WHERE `user`.`USER_ID` = "'.$user_id.'";';

		$db = $this->getDatabase();
		$result = $db->query( $query );
		if( $result )
		{
			if( $db->numRows($result) > 0 )
			{
				$row = $db->fetch( $result, Database::F_ASSOC );
				return $row;
			}
		}

		return array();
	}

	/**
	 * Modifica la información de un usuario
	 * @param string $user_id
	 * @param array $data
	 * @return boolean
	 */
	public function modifyUser( $user_id, $data )
	{
		$db = $this->getDatabase();

		$separator = '';
		foreach( $data as $field => $value )
		{
			if( $field == 'USER_PASSWORD')
			{
				$value = md5( $value );
			}
			$fields .= $separator.' `'.$field.'` = "'.$db->evalSQL($value).'"';
			$separator = ',';
		}

		$query = 'UPDATE '.$this->user_table.' SET '.$fields.' WHERE `USER_ID` = "'.$user_id.'" LIMIT 1;';

		if( !$db->query( $query ) )
		{
			return false;
		}

		return true;
	}

	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************

	/**
	 * Obtiene la base de datos para usuarios
	 * @return Database
	 */
	private function getDatabase()
	{
		$db = Database::getInstance();
		
		if( $db->selectProfile( $this->connection_profile ) )
		{
			return $db;	
		}
		else
		{
			trigger_error( 'No se ha podido establecer la conexión con la base de datos de usuarios', E_USER_WARNING );
			return NULL;
		}
	}
	
	/**
	 * Obtiene los campos adicionales para inclurilos en la query
	 * @return string
	 * @final
	 */
	final private function otherData()
	{
		$query = '';
		if( !empty( $this->data ))
		{
			$db = $this->getDatabase();
			foreach( $this->data as $field )
			{
				$field = $db->evalSQL( $field );
				$query .= ', `'.$field.'`';
			}
		}

		return $query;
	}

	/**
	 * Modifica el nivel de la sesión del usuario
	 * @param int $level
	 * @return void
	 * @final
	 */
	final private function setLevel( $level )
	{
		Session::set( 'level', $level );
	}

	/**
	 * Asegura los datos de sesión para que no sean modificados
	 * @return void
	 * @final
	 */
	final private function lockData()
	{
		$strictdata = array( 'user_id', 'screenname', 'level' );
		$userdata = array_merge( $strictdata, $this->data );
		Session::lock( 'userdata', $userdata );
	}

	/**
	 * Guarda los datos de la sesión del usuario
	 * @param string $data
	 * @param string $value
	 * @return void
	 */
	protected function storeData( $data, $value )
	{
		Session::set( $data, $value );
	}

	/**
	 * Guarda todos los datos adicionales del usuario
	 * @param object $data
	 * @return void
	 */
	private function storeAllData( $data )
	{
		foreach( $this->data as $field )
		{
			if( $field = 'level' || $field = 'screenname' )
			{
				continue;
			}
			$this->storeData( $field, $data->$field );
		}
	}

	/**
	 * Borra todos los datos adicionales del usuario
	 * @return void
	 */
	private function clearAllData()
	{
		foreach( $this->data as $field )
		{
			$this->storeData( $field, '' );
		}
	}

	/**
	 * Agrega un campo adicional a los daotos del usuario
	 * @param string $field
	 * @return void
	 */
	protected function addDataField( $field )
	{
		$this->data[] = $field;
	}

	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************

	/**
	 * Método estático para registrar la clase como el manejador de usuarios activo
	 * @param string[optional] $connection_profile
	 * @return void
	 * @static
	 */
	public static function register( $connection_profile = 'default' )
	{
		$handler = new self;
		$handler->setConnectionProfile( $connection_profile );
		Registry::set( 'usermanager', $handler );
	}
}

?>
