<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * @file include/lib/core/functions.core.php
 * Funciones estándar de la librería Core
 *
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */

/**
 * Detiene la ejecución mostrando una página de error
 * @param string $error_string
 * @param string $error_file
 * @param int $error_line
 * @return void
 */
function md_halt( $error_string, $error_file, $error_line )
{
	$tpl = new Template( 'pages/error' );
	$buffer = Buffer::getInstance();
	$buffer->end();
	$system_warning = $buffer->getContent( 'e_warning' );
	$system_notice = $buffer->getContent( 'e_notice' );
	$tpl->setVars( array( 'title' => Vars::get( 'title' ), 'Error_Message' => $error_string, 'system_warning' => $system_warning, 'system_notice' => $system_notice ) );
	
	die( (string) $tpl );
}

/**
 * Agrega una variable a la vista
 * @param string $name
 * @param mixed $value
 * @return void
 */
function md_addVar( $name, $value )
{
	$view = Registry::get( 'view' );	
	if( $view instanceof View )
	{
		$view->addVar( $name, $value );
	}
	else
	{
		trigger_error( 'No hay una vista por defecto definida al intentar agregar una variable', E_USER_NOTICE );
	}
}

/**
 * Agrega una plantilla a la vista
 * @param string $name
 * @param string $template
 * @return void
 */
function md_addTemplate( $name, $template)
{
	$view = Registry::get( 'view' );
	if( $view instanceof HTMLView )
	{
		$view->addTemplate( $name, $template );
	}
	else
	{
		trigger_error( 'La vista por defecto no es HTMLView o no esta definida al intentar agregar una plantilla' );
	}
}

/**
 * Redirecciona la página a otra ubicación
 * @param string $location
 * @return void
 */
function md_redirect( $location )
{
	if( Registry::get( 'stop_redirection' ) )
	{
		trigger_error( 'No esta permitida la redirección en este momento', E_USER_WARNING ); // @todo localizar contenido
	}
	else
	{
		Registry::set( 'redirect_location', $location );
	}
}

/**
 * Desahabilita la redirección del sistema
 * @return void
 */
function md_stopRedirection()
{
	Registry::set( 'stop_redirection', true );
	$array = debug_backtrace();
	$file = $array[0]['file'];
	$linea = $array[0]['line'];
	trigger_error( 'Se ha inhabilitado la redirección en el archivo '.$file.' línea '.$linea, E_USER_NOTICE ); // @todo localizar contenido
}

?>
